package me.wazup.hideandseek.utils;

import java.lang.reflect.InvocationTargetException;
import org.bukkit.Bukkit;
import org.bukkit.inventory.meta.PotionMeta;
import org.bukkit.potion.PotionType;
import org.bukkit.inventory.meta.EnchantmentStorageMeta;
import org.bukkit.enchantments.Enchantment;
import org.bukkit.Material;
import java.util.ArrayList;
import java.util.List;
import org.bukkit.inventory.meta.ItemMeta;
import org.bukkit.inventory.ItemStack;
import java.lang.reflect.Method;

public class ItemStackBuilder
{
    public static Method setDamageMethod;
    private final ItemStack item;
    private ItemMeta meta;
    private final List<String> lore;
    
    public ItemStackBuilder(final ItemStack item) {
        this.item = item;
        this.meta = item.getItemMeta();
        this.lore = ((this.meta != null && this.meta.hasLore()) ? this.meta.getLore() : new ArrayList<String>());
    }
    
    public ItemStackBuilder(final Material material) {
        this(new ItemStack(material));
    }
    
    public static void loadMethods() {
        try {
            for (final Method setDamageMethod : Class.forName("org.bukkit.inventory.meta.Damageable").getMethods()) {
                if (setDamageMethod.getName().equals("setDamage")) {
                    ItemStackBuilder.setDamageMethod = setDamageMethod;
                    break;
                }
            }
        }
        catch (final Exception ex) {}
    }
    
    public ItemStackBuilder setType(final Material type) {
        this.item.setType(type);
        return this;
    }
    
    public ItemStackBuilder setName(final String displayName) {
        this.meta.setDisplayName(displayName);
        return this;
    }
    
    public ItemStackBuilder addLore(final String... array) {
        for (int length = array.length, i = 0; i < length; ++i) {
            this.lore.add(array[i]);
        }
        return this;
    }
    
    public ItemStackBuilder clearLore() {
        this.lore.clear();
        return this;
    }
    
    public ItemStackBuilder addEnchantment(final Enchantment enchantment, final int n) {
        if (this.item.getType().equals((Object)Material.ENCHANTED_BOOK)) {
            ((EnchantmentStorageMeta)this.meta).addStoredEnchant(enchantment, n, true);
        }
        else {
            this.meta.addEnchant(enchantment, n, true);
        }
        return this;
    }
    
    public ItemStackBuilder setAmount(final int amount) {
        this.item.setAmount(amount);
        return this;
    }
    
    public ItemStackBuilder setDurability(final int n) {
        if (ItemStackBuilder.setDamageMethod != null) {
            try {
                ItemStackBuilder.setDamageMethod.invoke(this.meta, (short)n);
            }
            catch (final Exception ex) {
                ex.printStackTrace();
            }
        }
        else {
            this.item.setDurability((short)n);
        }
        return this;
    }
    
    public void setPotionEffect(final PotionType potionType, final boolean b, final boolean b2) {
        final PotionMeta potionMeta = (PotionMeta)this.item.getItemMeta();
        try {
            PotionMeta.class.getMethod("setBasePotionData", Class.forName("org.bukkit.potion.PotionData")).invoke(potionMeta, Class.forName("org.bukkit.potion.PotionData").getConstructor(PotionType.class, Boolean.TYPE, Boolean.TYPE).newInstance(potionType, b, b2));
        }
        catch (final IllegalAccessException | IllegalArgumentException | InvocationTargetException | NoSuchMethodException | SecurityException | ClassNotFoundException | InstantiationException ex) {
            Bukkit.getConsoleSender().sendMessage("[Hide and Seek] The used format for potions does not work on this minecraft version, please try other format which is POTION:DURABILITY");
        }
        this.meta = (ItemMeta)potionMeta;
    }
    
    public ItemStack build() {
        if (this.meta != null) {
            this.meta.setLore((List)this.lore);
        }
        this.item.setItemMeta(this.meta);
        return this.item;
    }
}
